<?php

/**
 * Copyright 2019 IQRF Tech s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
declare(strict_types = 1);

namespace Iqrf\Repository\Entities;

use JsonSerializable;

/**
 * IQRF OS version entity
 */
final class Os implements JsonSerializable {

	/**
	 * @var OsDpaAttributtes Attributes
	 */
	private $attributes;

	/**
	 * @var string IQRF OS build
	 */
	private $build;

	/**
	 * @var string Download path
	 */
	private $downloadPath;

	/**
	 * @var string IQRF OS version
	 */
	private $version;

	/**
	 * Constructor
	 * @param string $build IQRF OS build
	 * @param string $version IQRF OS version
	 * @param OsDpaAttributtes $attributes Attributes
	 * @param string $downloadPath Download path
	 */
	public function __construct(string $build, string $version, OsDpaAttributtes $attributes, string $downloadPath) {
		$this->attributes = $attributes;
		$this->build = $build;
		$this->downloadPath = $downloadPath;
		$this->version = $version;
	}

	/**
	 * Creates a new IQRF OS entity from an array
	 * @param array<string, string|int> $data JSON data in an array
	 * @return Os Created IQRF OS entity
	 */
	public static function fromArray(array $data): self {
		return new self($data['build'], $data['version'], OsDpaAttributtes::fromInt($data['attributes']), $data['downloadPath']);
	}

	/**
	 * Returns IQRF OS version attributes
	 * @return OsDpaAttributtes IQRF OS version attributes
	 */
	public function getAttributes(): OsDpaAttributtes {
		return $this->attributes;
	}

	/**
	 * Returns the IQRF OS build
	 * @return string IQRF OS build
	 */
	public function getBuild(): string {
		return $this->build;
	}

	/**
	 * Returns the download path
	 * @return string Download path
	 */
	public function getDownloadPath(): string {
		return $this->downloadPath;
	}

	/**
	 * Returns the IQRF OS version
	 * @return string IQRF OS version
	 */
	public function getVersion(): string {
		return $this->version;
	}

	/**
	 * Serialized the IQRF OS entity into JSON
	 * @return array<string, string|array<string, bool|null>> JSON serialized IQRF OS entity
	 */
	public function jsonSerialize(): array {
		return [
			'build' => $this->build,
			'version' => $this->version,
			'attributes' => $this->attributes->jsonSerialize(),
			'downloadPath' => $this->downloadPath,
		];
	}

}
